@@ -13,7 +13,8 @@ use distribution_types::{
 };
 use platform_tags::Tags;
 use pypi_types::{Requirement, RequirementSource, ResolverMarkerEnvironment};
-use uv_cache::{ArchiveTimestamp, Cache, CacheBucket, WheelCache};
+use uv_cache::{Cache, CacheBucket, WheelCache};
+use uv_cache_info::{CacheInfo, Timestamp};
 use uv_configuration::{BuildOptions, Reinstall};
 use uv_distribution::{
     BuiltWheelIndex, HttpArchivePointer, LocalArchivePointer, RegistryWheelIndex,
@@ -93,8 +94,7 @@ impl<'a> Planner<'a> {
                 }
             }
 
-            // Check if the package should be reinstalled. A reinstall involves (1) purging any
-            // cached distributions, and (2) marking any installed distributions as extraneous.
+            // Check if the package should be reinstalled.
             let reinstall = match reinstall {
                 Reinstall::None => false,
                 Reinstall::All => true,
@@ -207,6 +207,7 @@ impl<'a> Planner<'a> {
                                         wheel.filename,
                                         wheel.url,
                                         archive.hashes,
+                                        CacheInfo::default(),
                                         cache.archive(&archive.id),
                                     );
 
@@ -361,14 +362,16 @@ impl<'a> Planner<'a> {
                                 .entry(format!("{}.rev", wheel.filename.stem()));
 
                             if let Some(pointer) = LocalArchivePointer::read_from(&cache_entry)? {
-                                let timestamp = ArchiveTimestamp::from_file(&wheel.install_path)?;
+                                let timestamp = Timestamp::from_path(&wheel.install_path)?;
                                 if pointer.is_up_to_date(timestamp) {
+                                    let cache_info = pointer.to_cache_info();
                                     let archive = pointer.into_archive();
                                     if archive.satisfies(hasher.get(&wheel)) {
                                         let cached_dist = CachedDirectUrlDist::from_url(
                                             wheel.filename,
                                             wheel.url,
                                             archive.hashes,
+                                            cache_info,
                                             cache.archive(&archive.id),
                                         );
 
