@@ -18,7 +18,8 @@ use pep508_rs::{MarkerEnvironment, StringVersion};
 use platform_tags::Platform;
 use platform_tags::{Tags, TagsError};
 use pypi_types::{ResolverMarkerEnvironment, Scheme};
-use uv_cache::{Cache, CacheBucket, CachedByTimestamp, Freshness, Timestamp};
+use uv_cache::{Cache, CacheBucket, CachedByTimestamp, Freshness};
+use uv_cache_info::Timestamp;
 use uv_fs::{write_atomic_sync, PythonExt, Simplified};
 
 use crate::implementation::LenientImplementationName;
