@@ -282,7 +282,9 @@ impl ManagedPythonInstallation {
 
     /// The [`PythonVersion`] of the toolchain.
     pub fn version(&self) -> PythonVersion {
-        self.key.version()
+        self.key
+            .version()
+            .expect("Managed Python installations should always have valid versions")
     }
 
     pub fn implementation(&self) -> &ImplementationName {
@@ -329,13 +331,17 @@ impl ManagedPythonInstallation {
         let stdlib = if matches!(self.key.os, Os(target_lexicon::OperatingSystem::Windows)) {
             self.python_dir().join("Lib")
         } else {
+            let version = self
+                .key
+                .version()
+                .expect("Managed Python installations should always have valid versions");
             let python = if matches!(
                 self.key.implementation,
                 LenientImplementationName::Known(ImplementationName::PyPy)
             ) {
-                format!("pypy{}", self.key.version().python_version())
+                format!("pypy{}", version.python_version())
             } else {
-                format!("python{}", self.key.version().python_version())
+                format!("python{}", version.python_version())
             };
             self.python_dir().join("lib").join(python)
         };
