@@ -50,7 +50,7 @@ pub(crate) async fn init(
     if path.join("pyproject.toml").exists() {
         let path = std::path::absolute(&path).unwrap_or_else(|_| path.simplified().to_path_buf());
         anyhow::bail!(
-            "Project is already initialized in `{}`",
+            "Project is already initialized in `{}` (`pyproject.toml` file exists)",
             path.display().cyan()
         );
     }
@@ -471,7 +471,7 @@ impl InitProjectKind {
                 fs_err::write(
                     init_py,
                     indoc::formatdoc! {r#"
-                    def hello():
+                    def hello() -> None:
                         print("Hello from {name}!")
                     "#},
                 )?;
