@@ -957,6 +957,8 @@ pub(crate) struct ExportSettings {
     pub(crate) extras: ExtrasSpecification,
     pub(crate) dev: bool,
     pub(crate) hashes: bool,
+    pub(crate) install_options: InstallOptions,
+    pub(crate) output_file: Option<PathBuf>,
     pub(crate) locked: bool,
     pub(crate) frozen: bool,
     pub(crate) python: Option<String>,
@@ -978,6 +980,10 @@ impl ExportSettings {
             no_dev,
             hashes,
             no_hashes,
+            output_file,
+            no_emit_project,
+            no_emit_workspace,
+            no_emit_package,
             locked,
             frozen,
             resolver,
@@ -995,6 +1001,12 @@ impl ExportSettings {
             ),
             dev: flag(dev, no_dev).unwrap_or(true),
             hashes: flag(hashes, no_hashes).unwrap_or(true),
+            install_options: InstallOptions::new(
+                no_emit_project,
+                no_emit_workspace,
+                no_emit_package,
+            ),
+            output_file,
             locked,
             frozen,
             python,
